#pragma once

#include <string>
#include <vector>

#include <Poco/Thread.h>
#include <Poco/RunnableAdapter.h>
#include <Poco/Process.h>
#include <Poco/SharedPtr.h>
#include <Poco/PipeStream.h>

#include "Constants.h"

#ifdef WIN32

#ifdef FASPSTREAM_CPP_EXPORTS
#define FASPSTREAM_CPP_API __declspec(dllexport)
#else
#define FASPSTREAM_CPP_API __declspec(dllimport)
#endif

#else // NOT WIN32
#define FASPSTREAM_CPP_API
#endif

class FaspOutputStreamToFile
{

private:
    //  Private members are mostly configuration settings, set at the client (sender)
    Poco::SharedPtr<Poco::ProcessHandle> _ph;
    Poco::Pipe _inPipe;
    Poco::Pipe _errPipe;

    bool _lockPolicy;
    bool _lockMinimum;
    bool _lockTarget;
    int _tcpPort;
    int _udpPort;
    int _datagramSize;
    int _chunkSize;
    std::string _username;
    std::string _password;
    std::string _hostname;
    int _targetRate;
    FASPSTREAM_CIPHER_TYPE _cipherType;
    long _fileSize;
    int _minRate;
    int _maxMemory;
    std::string _ascpPath;
    FASPSTREAM_POLICY_TYPE _policy;


    std::string _errorString;
    bool _subprocessDied;
    std::string _destinationFilename;
    std::string _destinationDirectory;
    std::string _logDirectory;
    std::string _sshKeyFilePath;
    std::string _license;

    const std::vector<std::string> getCommandLine();

public:
    FASPSTREAM_CPP_API FaspOutputStreamToFile();
    FASPSTREAM_CPP_API ~FaspOutputStreamToFile();

    //  Should be a private method, but package scope for testability, not in interface
    FASPSTREAM_CPP_API std::vector<std::string> getCommandline() const;

    FASPSTREAM_CPP_API bool connect(long size);
    FASPSTREAM_CPP_API std::string getUsername() const;
    FASPSTREAM_CPP_API void setUsername(const std::string& username);
    FASPSTREAM_CPP_API std::string getPassword() const;
    FASPSTREAM_CPP_API void setPassword(const std::string& password);
    FASPSTREAM_CPP_API int getMaxMemory() const;
    FASPSTREAM_CPP_API void setMaxMemory(int maxMem);
    FASPSTREAM_CPP_API int getUdpPort() const;
    FASPSTREAM_CPP_API void setUdpPort(int udpPort);
    FASPSTREAM_CPP_API int getTcpPort() const;
    FASPSTREAM_CPP_API void setTcpPort(int tcpPort);
    FASPSTREAM_CPP_API int getDatagramSize() const;
    FASPSTREAM_CPP_API void setDatagramSize(int size);
    FASPSTREAM_CPP_API int getChunkSize() const;
    FASPSTREAM_CPP_API void setChunkSize(int size);
    FASPSTREAM_CPP_API std::string getHostname() const;
    FASPSTREAM_CPP_API void setHostname(const std::string& hostname);
    FASPSTREAM_CPP_API int getTargetRate() const ;
    FASPSTREAM_CPP_API void setTargetRate(int target);
    FASPSTREAM_CPP_API FASPSTREAM_CIPHER_TYPE getCipherType() const;
    FASPSTREAM_CPP_API void setCipherType(FASPSTREAM_CIPHER_TYPE type);
    FASPSTREAM_CPP_API bool isLockTarget() const;
    FASPSTREAM_CPP_API void setLockTarget(bool target);
    FASPSTREAM_CPP_API int getMinRate() const;
    FASPSTREAM_CPP_API void setMinRate(int minimum);
    FASPSTREAM_CPP_API bool isLockMinimum() const;
    FASPSTREAM_CPP_API void setLockMinimum(bool lockMin);
    FASPSTREAM_CPP_API FASPSTREAM_POLICY_TYPE getPolicy() const;
    FASPSTREAM_CPP_API void setPolicy(FASPSTREAM_POLICY_TYPE newPolicy);
    FASPSTREAM_CPP_API bool isLockPolicy() const;
    FASPSTREAM_CPP_API void setLockPolicy(bool lock);
    FASPSTREAM_CPP_API int getAggressiveness() const;
    FASPSTREAM_CPP_API void setAggressiveness(int aggressive);
    FASPSTREAM_CPP_API void write(const char* buffer, int length);
    FASPSTREAM_CPP_API std::string close();
    FASPSTREAM_CPP_API void forceClose();
    FASPSTREAM_CPP_API std::string getAscpPath() const;
    FASPSTREAM_CPP_API void setAscpPath(const std::string& ascpPath);
    FASPSTREAM_CPP_API std::string getDestinationFilename() const;
    FASPSTREAM_CPP_API void setDestinationFilename(
        const std::string& destinationFilename);
    FASPSTREAM_CPP_API std::string getDestinationDirectory() const;
    FASPSTREAM_CPP_API void setDestinationDirectory(
        const std::string& destinationDirectory);
    FASPSTREAM_CPP_API std::string getLogDirectory() const;
    FASPSTREAM_CPP_API void setLogDirectory(const std::string& logDirectory);
    FASPSTREAM_CPP_API std::string getSshKeyFilePath() const;
    FASPSTREAM_CPP_API void setSshKeyFilePath(
        const std::string& sshKeyFilePath);
    FASPSTREAM_CPP_API long getFileSize() const;
    FASPSTREAM_CPP_API void setFileSize(const long fileSize);
    void handleError(int errorCode);
    FASPSTREAM_CPP_API void setLicense(std::string commandToExecute);
    FASPSTREAM_CPP_API std::string getLicense() const;

};
