/*
 * Aspera, Inc.
 *
 * C version of FaspConnectionChannel interface.
 *
 * We do this for portability - the wrapper abstracts the socket used
 * to construct the channel. See below.
 */

#ifndef ASPERA_FASP_STREAM_CHANNEL_H
#define ASPERA_FASP_STREAM_CHANNEL_H

#include <stdlib.h>

#ifdef WIN32

#ifdef FASPSTREAM_CPP_EXPORTS
#define FASPSTREAM_C_API __declspec(dllexport)
#else
#define FASPSTREAM_C_API __declspec(dllimport)
#endif

#else /* NOT WIN32 */
#define FASPSTREAM_C_API
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
 * This is used to abstract socket differences between Unix, Windows, and
 * other platforms.
 *
 * IMPORTANT: Ensure the socket you use is set up for timeout with
 *            FASPSTREAM_CHANNEL_TIMEOUT as the timeout (defined in
 *            fasp_stream_constants.h).
 */
struct fasp_stream_socket_wrapper {
    /**
     * Private data/context your code will need in order to process
     * read/write. For example, on Linux, private should point to the socket
     * file descriptor, or to an as_feed_t.
     */
    void *priv;
    /**
     * This is invoked when the feed should read data from the socket.
     *
     * Return number of bytes read.
     *
     * Return zero for EOF or time out. For time out, set timedout to non-zero.
     *
     * Return negative on error.
     */
    int (*read)(struct fasp_stream_socket_wrapper *w, char *buff, int len,
        int *timedout);
    /**
     * This is invoked when the feed should write data to the socket.
     *
     * Return number of bytes written, negative on error.
     */
    int (*write)(struct fasp_stream_socket_wrapper *w, const char *buff,
        int len);
    /**
     * This is invoked when fasp stream is finished sharing the TCP
     * connection.
     */
    void (*close)(struct fasp_stream_socket_wrapper *w);
};
typedef struct fasp_stream_socket_wrapper fasp_stream_socket_wrapper_t;

#ifdef __cplusplus
}
#endif

#endif
