/*
 * Aspera, Inc.
 *
 * Enums for FaspStream transfer settings.
 */

#ifndef ASPERA_FASP_STREAM_ENUMS_H
#define ASPERA_FASP_STREAM_ENUMS_H

#if defined(_WIN64) || defined(_WIN32)
#define FASPSTREAM_BINARY "faspstream.exe"
#else
#define FASPSTREAM_BINARY "./faspstream"
#endif

#if defined(_WIN64) || defined(_WIN32)
#define ASCP_BINARY "ascp.exe"
#else
#define ASCP_BINARY "./ascp"
#endif

/**
 * Congestion control policy.
 */
enum FASPSTREAM_POLICY_TYPE {
    FASPSTREAM_POLICY_TYPE_FAIR = 0,
    FASPSTREAM_POLICY_TYPE_FIXED,
    FASPSTREAM_POLICY_TYPE_HIGH,
    FASPSTREAM_POLICY_TYPE_LOW,
    FASPSTREAM_POLICY_TYPE_TRICKLE,
    FASPSTREAM_POLICY_TYPE_UNINITIALIZED
};

/**
 * Encryption cypher type.
 */
enum FASPSTREAM_CIPHER_TYPE {
    FASPSTREAM_CIPHER_TYPE_NONE = 0,
    FASPSTREAM_CIPHER_TYPE_AES128,
    FASPSTREAM_CIPHER_TYPE_UNINITIALIZED
};

/**
 * Transfer mode - send or receive.
 */
enum FASPSTREAM_TRANSFER_MODE_TYPE {
    FASPSTREAM_TRANSFER_MODE_TYPE_UNINITIALIZED = 0,
    FASPSTREAM_TRANSFER_MODE_TYPE_DIRECT_SEND,
    FASPSTREAM_TRANSFER_MODE_TYPE_DIRECT_RECEIVE,
    FASPSTREAM_TRANSFER_MODE_TYPE_SSH_SEND,
    FASPSTREAM_TRANSFER_MODE_TYPE_SSH_RECEIVE
};

/**  Function type definition for registering a logging callback
  *
  *  Provide your own logging function by registering a funcion
  *  of this type (void return, const char* argument) with the
  *  faspstream client or server.  (Will not log if NULL, by default)
  *
  *  The simplest implementation of the callback might look like this:
  *  <code><br/><br/>
  *      static void log_function( char const *s ) {<br/>
  *  &nbsp;  fprintf( stderr, "%s\n", s );<br/>
  *      }
  *  </code>
  */
typedef void (*log_callback_f)( char const *s );

/**
 * Timeout used for tunneling, in microseconds.
 */
#define FASPSTREAM_CHANNEL_TIMEOUT 100000

#endif
