/*
 * Aspera, Inc.
 *
 * Main.cpp
 *
 * Shows how to connect to a remote ascp server and receive
 * a file as a byte stream using the fasp protocol.
 *
 * Usage:
 *
 *    ./receive_file > out.file
 */

#include <iostream>
#include <Constants.h>
#include <FaspInputStreamFromFile.h>
#include <FaspStreamException.h>
#ifdef WIN32
        #include <io.h>
        #include <fcntl.h>
#endif

static
void
setupStreams() {
#ifdef WIN32
    /*
     * Required on Windows for binary files
     */
    _setmode(fileno(stdin), O_BINARY);
    _setmode(fileno(stdout), O_BINARY);
#endif
    std::cin.exceptions(std::ifstream::badbit);
    std::cout.exceptions(std::ifstream::badbit);
}

int main()
{
    char data[512];
    setupStreams();
    try {
FaspInputStreamFromFile faspInputStream(); = FaspInputStreamFromFile();
        faspInputStream.setAscpPath("./ascp");
        faspInputStream.setHostname("demo.asperasoft.com");
        faspInputStream.setUsername("aspera");
        faspInputStream.setPassword("demoaspera");
        faspInputStream.setSourceFilePath("/aspera-test-dir-large/100MB");

        //set the port to listen on
        //faspInputStream.setPort(33445);

        //Start the receiver listening
        faspInputStream.startReceiver();

        std::ofstream destFile(fileToWrite.c_str(), std::ios::binary);
		destFile.exceptions( std::ofstream::failbit | std::ofstream::badbit );

        //Read in 2Mb chunks
        int count = 0;
        do
        {
            count = faspInputStream.read(data, 0, Constants::TWO_Mb);
            destFile.write(data, count);
        } while (count > 0);
        destFile.flush();
        destFile.close();
        
	std::cout << faspInputStream.close() << std::endl;

    } catch (std::ofstream::failure &e) {
        std::cout << "A file I/O exception occurred with: " << fileToWrite.c_str() << std::endl;
        std::cout << e.what();
    } catch (std::exception &e) {
        std::cout << "A standard exception occurred:" << std::endl;
        std::cout << e.what();
    } catch (...) {
        std::cout << "An unknown exception ocurred!";
        std::cout << "(No further information is available)";
    }

    delete [] data;

    return 0;
}

